@@ -17,8 +17,8 @@
    with this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#include "system.h"
-#include "rmt.h"
+#include <system.h>
+#include <rmt.h>
 #include "common.h"
 #include <quotearg.h>
 #include <save-cwd.h>
@@ -317,7 +317,8 @@ remove_any_file (const char *file_name, enum remove_option option)
 		 entry += entrylen + 1)
 	      {
 		char *file_name_buffer = new_name (file_name, entry);
-		int r = remove_any_file (file_name_buffer, 1);
+		int r = remove_any_file (file_name_buffer, 
+                                         RECURSIVE_REMOVE_OPTION);
 		int e = errno;
 		free (file_name_buffer);
 
